#include "cyc_cnt.h"
#include "lcd.h"
#include "leds.h"
#include "uart.h"
#include "switches.h"

#define CFG_DELAY
#define CFG_LCD
#define CFG_UART
//#define CFG_UART_CHK

const int myconst = 0x12345678;

int myvar = 0x11223344;

volatile int data[100];

void switches(void)
{
#ifdef CFG_LCD
  lcd_chr(1, 0, switches_get_state(sw_0) ? '0' : ' ');
  lcd_chr(1, 1, switches_get_state(sw_1) ? '1' : ' ');
  lcd_chr(1, 2, switches_get_state(sw_2) ? '2' : ' ');
  lcd_chr(1, 3, switches_get_state(sw_3) ? '3' : ' ');
  lcd_chr(1, 4, switches_get_state(sw_east) ? 'E' : ' ');
  lcd_chr(1, 5, switches_get_state(sw_north) ? 'N' : ' ');
  lcd_chr(1, 6, switches_get_state(sw_south) ? 'S' : ' ');
  lcd_chr(1, 7, switches_get_state(sw_west) ? 'W' : ' ');
  lcd_chr(1, 8, switches_get_state(sw_center) ? 'C' : ' ');
  lcd_chr(1, 9, switches_get_state(sw_rot_a) ? 'a' : ' ');
  lcd_chr(1, 10, switches_get_state(sw_rot_b) ? 'b' : ' ');
#endif

#ifdef CFG_LCD
  unsigned int cnt = switches_get_rot_cnt();
  lcd_chr(1, 12, '0' + (cnt >> 9 & 0x7));
  lcd_chr(1, 13, '0' + (cnt >> 6 & 0x7));
  lcd_chr(1, 14, '0' + (cnt >> 3 & 0x7));
  lcd_chr(1, 15, '0' + (cnt & 0x7));
#endif
}

void delay(void)
{
  unsigned int i;
  for (i = 0; i < 10; ++i) {
    switches();
#ifdef CFG_DELAY
    cyc_cnt_delay_ms(20);
#endif
  }
}

int main()
{
  unsigned int i;
  unsigned char leds;

  for (i = 0; i < sizeof(data) / sizeof(data[0]); ++i)
    data[i] = i;

#ifdef CFG_LCD
  lcd_init();
  lcd_str(0, "MIPS I system");
  lcd_str(1, "");
#endif

#ifdef CFG_UART
  uart_cfg_scale(62); /* 115200 */
  uart_cfg_bits(8);
  uart_cfg_stop(1);
  uart_tx('M');
  uart_tx('I');
  uart_tx('P');
  uart_tx('S');
  uart_tx(' ');
  uart_tx('I');
  uart_tx('\r');
  uart_tx('\n');
#ifdef CFG_UART_CHK
  if (uart_rx() != 'M') while(1);
  if (uart_rx() != 'I') while(1);
  if (uart_rx() != 'P') while(1);
  if (uart_rx() != 'S') while(1);
  if (uart_rx() != ' ') while(1);
  if (uart_rx() != 'I') while(1);
  if (uart_rx() != '\r') while(1);
  if (uart_rx() != '\n') while(1);
#endif
#endif

  leds = 0x11;
  while (1) {
#ifdef CFG_UART
    if (uart_can_rx())
      leds = uart_rx();
#endif
    leds_set_state(leds);
    leds = leds << 1 | leds >> 7;
    delay();
  }

  return 0;
}

